==> home-made-io-fastmag-sync-m2-0.24.0-762a73.zip: Process/Worker/ToMagento/Integration/Order/Status.php <==
<?php
/*******************************************************************************
 * Created by HOMEMADE.IO SAS.
 * Fastmag Sync  -  Connect Fastmag with your Magento
 *
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author    Simon Laubet-Xavier <simon.laubetxavier@home-made.io>
 * @copyright 2020-2021 HOMEMADE.IO SAS
 * @date      2021-09-30
 ******************************************************************************/

namespace Fastmag\Sync\Process\Worker\ToMagento\Integration\Order;

use Fastmag\Sync\Api\Data\Jobqueue\ToMagentoInterface as Job;
use Fastmag\Sync\Api\Data\Rule\CarrierInterface as CarrierRule;
use Fastmag\Sync\Api\Jobqueue\ToMagentoRepositoryInterface as JobRepository;
use Fastmag\Sync\Api\OrderRepositoryInterface as SyncedOrderRepository;
use Fastmag\Sync\Api\Rule\CarrierRepositoryInterface as CarrierRuleRepository;
use Fastmag\Sync\Exception\JobException;
use Fastmag\Sync\Exception\ProcessException;
use Fastmag\Sync\Logger\Logger;
use Fastmag\Sync\Model\Config;
use Fastmag\Sync\Process\Entity\ToMagento\Order as OrderEntity;
use Fastmag\Sync\Process\Worker\ToMagento\Integration\Order;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\MailException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Api\Data\ShipmentTrackInterface as Track;
use Magento\Sales\Api\OrderRepositoryInterface as OrderRepository;
use Magento\Sales\Api\ShipmentRepositoryInterface as ShipmentRepository;
use Magento\Sales\Exception\CouldNotShipException;
use Magento\Sales\Model\Convert\Order as OrderConverter;
use Magento\Sales\Model\Order as OrderModel;
use Magento\Sales\Model\Order\Shipment as ShipmentModel;
use Magento\Sales\Model\Order\Shipment\TrackFactory;
use Magento\Shipping\Model\ShipmentNotifier;
use Magento\Store\Model\ScopeInterface;

/**
 * Class Status
 *
 * Integration worker for tomagento_integration_order_status jobs
 */
class Status extends Order
{
    /** @inheritDoc */
    protected $code = 'tomagento_integration_order_status';

    /** @var OrderConverter $orderConverter */
    protected $orderConverter;

    /** @var TrackFactory $trackFactory */
    protected $trackFactory;

    /** @var ShipmentRepository $shipmentRepository */
    protected $shipmentRepository;

    /** @var ShipmentNotifier $shipmentNotifier */
    protected $shipmentNotifier;

    /** @var CarrierRuleRepository $carrierRuleRepository */
    protected $carrierRuleRepository;

    /** @var SearchCriteriaBuilder $searchCriteriaBuilder */
    protected $searchCriteriaBuilder;

    /** @var string $hydrationWorker */
    protected $hydrationWorker = 'tomagento_hydration_order';

    /** @var Job $currentJob */
    protected $currentJob;

    /**
     * Status constructor
     *
     * @param Logger                $logger
     * @param JobRepository         $jobRepository
     * @param Config                $config
     * @param SyncedOrderRepository $syncedOrderRepository
     * @param OrderRepository       $orderRepository
     * @param OrderConverter        $orderConverter
     * @param TrackFactory          $trackFactory
     * @param ShipmentRepository    $shipmentRepository
     * @param ShipmentNotifier      $shipmentNotifier
     * @param CarrierRuleRepository $carrierRuleRepository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     */
    public function __construct(
        Logger $logger,
        JobRepository $jobRepository,
        Config $config,
        SyncedOrderRepository $syncedOrderRepository,
        OrderRepository $orderRepository,
        OrderConverter $orderConverter,
        TrackFactory $trackFactory,
        ShipmentRepository $shipmentRepository,
        ShipmentNotifier $shipmentNotifier,
        CarrierRuleRepository $carrierRuleRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder
    ) {
        parent::__construct($logger, $jobRepository, $config, $syncedOrderRepository, $orderRepository);

        $this->orderConverter = $orderConverter;
        $this->trackFactory = $trackFactory;
        $this->shipmentRepository = $shipmentRepository;
        $this->shipmentNotifier = $shipmentNotifier;
        $this->carrierRuleRepository = $carrierRuleRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }

    /**
     * @inheritDoc
     */
    public function run()
    {
        try {
            $this->initiate();
        } catch (ProcessException $exception) {
            $this->logger->notice($exception->getMessage());

            return;
        }

        foreach ($this->getJobs()->getItems() as $job) {
            if (!$job->isInError()) {
                try {
                    $this->processJob($job);
                } catch (JobException $exception) {
                    $this->invalidateJob($job, $exception);
                } catch (NoSuchEntityException | CouldNotShipException $exception) {
                    $this->skipJob($job, $exception);
                }

                $this->saveJob($job);
            }
        }
    }

    /**
     * @inheritDoc
     *
     * @throws NoSuchEntityException
     * @throws CouldNotShipException
     */
    protected function processJob($job)
    {
        $this->currentJob = $job;
        /** @var OrderEntity $entity */
        $entity = $this->currentJob->getEntity();

        $order = $this->getMagentoOrder($entity->getFastmagId());

        if ($order->canShip() && $this->isOrderSent($entity->getStatus())) {
            $shipment = $this->createShipment($order);

            $track = $this->generateTrack($entity, $order);
            $shipment->addTrack($track);

            try {
                /** @throws CouldNotSaveException */
                $this->shipmentRepository->save($shipment);
            } catch (CouldNotSaveException $exception) {
                throw new JobException(__(
                    'Unable to save the shipment: %1',
                    ($exception->getPrevious() ? $exception->getPrevious()->getMessage() : $exception->getMessage())
                ));
            }

            $this->orderRepository->save($order);

            try {
                $this->shipmentNotifier->notify($shipment);

                $this->shipmentRepository->save($shipment);
            } catch (MailException $exception) {
                throw new JobException(
                    __('Unable to send notify the customer of the shipment: %1', $exception->getMessage())
                );
            }
        } else {
            throw new CouldNotShipException(__('Order can\'t be shipped'));
        }
    }

    /**
     * @inheritDoc
     */
    public function isEnabled()
    {
        return $this->config->isSetFlag(Config::XML_PATH_ORDER_IMPORT_STATUS_UPDATES);
    }

    /**
     * Returns Magento order object from transaction ID
     *
     * @param int $transactionId
     *
     * @return OrderModel
     *
     * @throws NoSuchEntityException
     */
    protected function getMagentoOrder($transactionId)
    {
        $syncedOrder = $this->syncedOrderRepository->getLastSyncByTransactionId($transactionId);

        $magentoId = $syncedOrder->getOrderId();

        return $this->orderRepository->get($magentoId);
    }

    /**
     * Tells if the order is considered as sent in Fastmag
     *
     * @param int $status
     *
     * @return bool
     */
    protected function isOrderSent($status)
    {
        return $status >= 300 && $status < 400;
    }

    /**
     * Generate track
     *
     * @param OrderEntity $entity
     * @param OrderModel  $order
     *
     * @return Track
     */
    protected function generateTrack($entity, $order)
    {
        $track = $this->trackFactory->create();

        $searchCriteria = $this->searchCriteriaBuilder
            ->addFilter(CarrierRule::FASTMAG_CODE, $entity->getCarrier())
            ->create();

        $carrierRules = $this->carrierRuleRepository->getList($searchCriteria);

        if ($carrierRules->getTotalCount() === 0) {
            $this->log($this->currentJob, 'No carrier rule found, using carrier of the original order');

            $shippingMethod = $order->getShippingMethod(true);
            $carrierCode = $shippingMethod->getData('carrier_code');
        } else {
            $carrierRulesItems = $carrierRules->getItems();

            /** @var CarrierRule $carrierRule */
            $carrierRule = array_shift($carrierRulesItems);
            $carrierCode = $carrierRule->getMethodCode();

            $this->log($this->currentJob, 'Carrier rule #' . $carrierRule->getId() . 'used');
        }

        $carrierTitle = $this->config->getValue(
            'carriers/' . $carrierCode . '/title',
            ScopeInterface::SCOPE_STORE,
            $order->getStoreId()
        );

        $track->setCarrierCode($carrierCode)
            ->setTitle($carrierTitle)
            ->setNumber($entity->getTrackingNumber());

        return $track;
    }

    /**
     * Create shipment from Magento order
     *
     * @param OrderModel $order
     *
     * @return ShipmentModel
     *
     * @throws JobException
     */
    protected function createShipment($order)
    {
        $shipment = $this->orderConverter->toShipment($order);

        foreach ($order->getAllItems() as $orderItem) {
            if (!$orderItem->getQtyToShip() || $orderItem->getIsVirtual()) {
                continue;
            }

            try {
                $shipmentItem = $this->orderConverter->itemToShipmentItem($orderItem)
                    ->setQty($orderItem->getQtyToShip());
            } catch (LocalizedException $exception) {
                throw new JobException(__('Unable to add item to shipment: %1', $exception->getMessage()));
            }

            $shipment->addItem($shipmentItem);
        }

        try {
            $shipment->register();
        } catch (LocalizedException $exception) {
            throw new JobException(__('Unable to save shipment: %1', $exception->getMessage()));
        }

        $shipment->getOrder()->setIsInProcess(true);

        return $shipment;
    }
}

==> home-made-io-fastmag-sync-m2-1.9.0-b17085.zip: Process/Worker/ToMagento/Integration/Order/Status.php <==
<?php
/*******************************************************************************
 * Created by HOMEMADE.IO SAS.
 * Fastmag Sync  -  Connect Fastmag with your Magento
 *
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author    Simon Laubet-Xavier <simon.laubetxavier@home-made.io>
 * @copyright 2020-2026 HOMEMADE.IO SAS
 * @date      2026-01-16
 ******************************************************************************/

namespace Fastmag\Sync\Process\Worker\ToMagento\Integration\Order;

use Fastmag\Sync\Api\Data\Jobqueue\ToMagentoInterface as Job;
use Fastmag\Sync\Api\Data\Rule\CarrierInterface as CarrierRule;
use Fastmag\Sync\Api\Jobqueue\ToMagentoRepositoryInterface as JobRepository;
use Fastmag\Sync\Api\OrderRepositoryInterface as SyncedOrderRepository;
use Fastmag\Sync\Api\Rule\CarrierRepositoryInterface as CarrierRuleRepository;
use Fastmag\Sync\Exception\JobException;
use Fastmag\Sync\Logger\Logger;
use Fastmag\Sync\Model\Config;
use Fastmag\Sync\Process\Entity\ToMagento\Order as OrderEntity;
use Fastmag\Sync\Process\Worker;
use Fastmag\Sync\Process\Worker\ToMagento\Integration as IntegrationTrait;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\MailException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Api\Data\ShipmentTrackInterface as Track;
use Magento\Sales\Api\OrderRepositoryInterface as OrderRepository;
use Magento\Sales\Api\ShipmentRepositoryInterface as ShipmentRepository;
use Magento\Sales\Model\Convert\Order as OrderConverter;
use Magento\Sales\Model\Order as OrderModel;
use Magento\Sales\Model\Order\Shipment as ShipmentModel;
use Magento\Sales\Model\Order\Shipment\TrackFactory;
use Magento\Shipping\Model\ShipmentNotifier;
use Magento\Store\Model\ScopeInterface;

/**
 * Class Status
 *
 * Integration worker for tomagento_integration_order_status jobs
 *
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 * @SuppressWarnings(PHPMD.LongVariable)
 */
class Status extends Worker
{
    use IntegrationTrait;

    /** @inheritDoc */
    protected $code = 'tomagento_integration_order_status';

    /** @var SyncedOrderRepository $syncedOrderRepository */
    protected $syncedOrderRepository;

    /** @var OrderRepository $orderRepository */
    protected $orderRepository;

    /** @var OrderConverter $orderConverter */
    protected $orderConverter;

    /** @var TrackFactory $trackFactory */
    protected $trackFactory;

    /** @var ShipmentRepository $shipmentRepository */
    protected $shipmentRepository;

    /** @var ShipmentNotifier $shipmentNotifier */
    protected $shipmentNotifier;

    /** @var CarrierRuleRepository $carrierRuleRepository */
    protected $carrierRuleRepository;

    /** @var SearchCriteriaBuilder $searchCriteriaBuilder */
    protected $searchCriteriaBuilder;

    /** @var string $hydrationWorker */
    protected $hydrationWorker = 'tomagento_hydration_order_status';

    /** @var string[] $subordinateWorkersAfter */
    protected $subordinateWorkersAfter = ['tomagento_integration_order_creditmemo'];

    /** @var Job $currentJob */
    protected $currentJob;

    /**
     * Status constructor
     *
     * @param Logger                $logger
     * @param JobRepository         $jobRepository
     * @param Config                $config
     * @param SyncedOrderRepository $syncedOrderRepository
     * @param OrderRepository       $orderRepository
     * @param OrderConverter        $orderConverter
     * @param TrackFactory          $trackFactory
     * @param ShipmentRepository    $shipmentRepository
     * @param ShipmentNotifier      $shipmentNotifier
     * @param CarrierRuleRepository $carrierRuleRepository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     *
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        Logger                $logger,
        JobRepository         $jobRepository,
        Config                $config,
        SyncedOrderRepository $syncedOrderRepository,
        OrderRepository       $orderRepository,
        OrderConverter        $orderConverter,
        TrackFactory          $trackFactory,
        ShipmentRepository    $shipmentRepository,
        ShipmentNotifier      $shipmentNotifier,
        CarrierRuleRepository $carrierRuleRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder
    ) {
        parent::__construct($logger);

        $this->jobRepository = $jobRepository;
        $this->config = $config;
        $this->syncedOrderRepository = $syncedOrderRepository;
        $this->orderRepository = $orderRepository;
        $this->orderConverter = $orderConverter;
        $this->trackFactory = $trackFactory;
        $this->shipmentRepository = $shipmentRepository;
        $this->shipmentNotifier = $shipmentNotifier;
        $this->carrierRuleRepository = $carrierRuleRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }

    /**
     * @inheritDoc
     */
    protected function processJob($job)
    {
        $this->currentJob = $job;
        /** @var OrderEntity $entity */
        $entity = $this->currentJob->getEntity();

        try {
            $order = $this->getMagentoOrder($entity->getFastmagId());
        } catch (NoSuchEntityException $exception) {
            $order = null;
        }

        if ($order && $entity instanceof OrderEntity && $this->isOrderSent((string)$entity->getStatus())) {
            if ($order->canShip()) {
                $shipment = $this->createShipment($entity, $order);

                try {
                    /** @throws CouldNotSaveException */
                    $this->shipmentRepository->save($shipment);
                    $this->debug($this->currentJob, 'Shipment #' . $shipment->getIncrementId() . ' created');
                } catch (CouldNotSaveException $exception) {
                    throw new JobException(__(
                        'Unable to save the shipment: %1',
                        ($exception->getPrevious() ? $exception->getPrevious()->getMessage() : $exception->getMessage())
                    ));
                }

                $this->orderRepository->save($order);
                $this->debug($this->currentJob, 'Order #' . $order->getIncrementId() . ' completed');

                $this->notifyShipment($shipment);
            } else {
                throw new JobException(__('Order can\'t be shipped'));
            }
        }
    }

    /**
     * @inheritDoc
     */
    public function isEnabled()
    {
        return $this->config->isSetFlag(Config::XML_PATH_ORDER_EXPORT_ENABLE)
            && ($this->config->isSetFlag(Config::XML_PATH_ORDER_IMPORT_STATUS_UPDATES)
            || $this->config->isSetFlag(Config::XML_PATH_ORDER_IMPORT_PRODUCTS_RETURNS));
    }

    /**
     * Returns Magento order object from transaction ID
     *
     * @param int $transactionId
     *
     * @return OrderModel
     *
     * @throws NoSuchEntityException
     */
    protected function getMagentoOrder($transactionId)
    {
        $syncedOrder = $this->syncedOrderRepository->getLastSyncByTransactionId($transactionId);

        $magentoId = $syncedOrder->getOrderId();

        return $this->orderRepository->get($magentoId);
    }

    /**
     * Tells if the order is considered as sent in Fastmag
     *
     * @param string $status
     *
     * @return bool
     */
    protected function isOrderSent(string $status): bool
    {
        $statusCode = (int)substr($status, 0, 3);

        return $statusCode >= 300 && $statusCode < 400;
    }

    /**
     * Generate track
     *
     * @param OrderEntity $entity
     * @param OrderModel  $order
     *
     * @return Track
     */
    protected function generateTrack($entity, $order)
    {
        $track = $this->trackFactory->create();

        $searchCriteria = $this->searchCriteriaBuilder
            ->addFilter(CarrierRule::FASTMAG_CODE, $entity->getCarrier())
            ->create();

        $carrierRules = $this->carrierRuleRepository->getList($searchCriteria);

        if ($carrierRules->getTotalCount() === 0) {
            $this->debug($this->currentJob, 'No carrier rule found, using carrier of the original order');

            $shippingMethod = $order->getShippingMethod(true);
            $carrierCode = $shippingMethod->getData('carrier_code');
        } else {
            $carrierRulesItems = $carrierRules->getItems();

            /** @var CarrierRule $carrierRule */
            $carrierRule = array_shift($carrierRulesItems);
            $carrierCode = $carrierRule->getMethodCode();

            $this->debug($this->currentJob, 'Carrier rule #' . $carrierRule->getId() . ' used');
        }

        $carrierTitle = $this->config->getValue(
            'carriers/' . $carrierCode . '/title',
            ScopeInterface::SCOPE_STORES,
            $order->getStoreId()
        );

        $track->setCarrierCode($carrierCode)
            ->setTitle($carrierTitle)
            ->setNumber($entity->getTrackingNumber());

        return $track;
    }

    /**
     * Create shipment from Magento order
     *
     * @param OrderEntity $entity
     * @param OrderModel  $order
     *
     * @return ShipmentModel
     *
     * @throws JobException
     */
    protected function createShipment($entity, $order)
    {
        $shipment = $this->orderConverter->toShipment($order);

        foreach ($order->getAllItems() as $orderItem) {
            if (!$orderItem->getQtyToShip() || $orderItem->getIsVirtual()) {
                continue;
            }

            try {
                $shipmentItem = $this->orderConverter->itemToShipmentItem($orderItem)
                    ->setQty($orderItem->getQtyToShip());
            } catch (LocalizedException $exception) {
                throw new JobException(__('Unable to add item to shipment: %1', $exception->getMessage()));
            }

            $shipment->addItem($shipmentItem);
        }

        try {
            $shipment->register();
        } catch (LocalizedException $exception) {
            throw new JobException(__('Unable to save shipment: %1', $exception->getMessage()));
        }

        $shipment->getOrder()->setIsInProcess(true);

        if ($entity->getTrackingNumber() !== null
            && $this->config->isSetFlag(Config::XML_PATH_ORDER_IMPORT_TRACKING_NUMBER)
        ) {
            $track = $this->generateTrack($entity, $order);
            $shipment->addTrack($track);
        }

        $shipmentExtension = $shipment->getExtensionAttributes();
        $shipmentExtension->setSourceCode($this->config->getValue(Config::XML_PATH_INVENTORY_MSI_SOURCE));
        $shipment->setExtensionAttributes($shipmentExtension);

        return $shipment;
    }

    /**
     * Notify customer of shipment creation
     *
     * @param ShipmentModel $shipment
     *
     * @return void
     *
     * @throws JobException
     */
    protected function notifyShipment($shipment)
    {
        try {
            $this->shipmentNotifier->notify($shipment);
        } catch (MailException $exception) {
            throw new JobException(
                __('Unable to send notify the customer of the shipment: %1', $exception->getMessage())
            );
        }
    }
}
